@@ -17,11 +17,10 @@
 This module manages configuration options
 """
 
-from oslo_config import cfg
-
 from neutron.conf.agent import common as agconfig
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.utils import net
+from oslo_config import cfg
 
 restproxy_opts = [
     cfg.ListOpt('servers', default=['localhost:8800'],
@@ -84,7 +83,10 @@ restproxy_opts = [
     cfg.BoolOpt('naming_scheme_unicode', default=True,
                 help=_("Configure whether or not to configure BCF "
                        "with unicode display-name. Applicable to BCF 5.0 "
-                       "onwards."))
+                       "onwards.")),
+    cfg.IntOpt('state_sync_interval', default=300,
+               help=_("Configure how often the OSP plugin should update "
+                      "cluster information on BCF"))
 ]
 router_opts = [
     cfg.MultiStrOpt('tenant_default_router_rule', default=['*:any:any:permit'],
