@@ -22,25 +22,23 @@ Big Switch core plugin.
 """
 import copy
 
-from oslo_log import helpers as log_helper
-from oslo_log import log as logging
-from oslo_utils import excutils
-from oslo_utils import uuidutils
-
 from neutron.api import extensions as neutron_extensions
-from neutron.db import api as db
 from neutron.db import dns_db
 from neutron.db import l3_db
-
-
 from neutron_lib.api.definitions import l3 as l3_apidef
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as lib_constants
+from neutron_lib.db import api as db_api
+from neutron_lib.db import utils as db_utils
 from neutron_lib import exceptions
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
+from oslo_log import helpers as log_helper
+from oslo_log import log as logging
+from oslo_utils import excutils
+from oslo_utils import uuidutils
 
 from networking_bigswitch.plugins.bigswitch.db import tenant_policy_db
 from networking_bigswitch.plugins.bigswitch import extensions
@@ -148,8 +146,6 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
                            resources.ROUTER, events.PRECOMMIT_DELETE)
         registry.subscribe(self.router_after_delete_callback, resources.ROUTER,
                            events.AFTER_DELETE)
-        registry.subscribe(self.router_interface_before_create_callback,
-                           resources.ROUTER_INTERFACE, events.BEFORE_CREATE)
         registry.subscribe(self.router_interface_after_create_callback,
                            resources.ROUTER_INTERFACE, events.AFTER_CREATE)
 
@@ -165,7 +161,7 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
         router = kwargs.get('router')
         self.txn_cache.add_transaction(router[BSN_TRANSACTION_ID],
                                        router['id'])
-        with db.context_manager.reader.using(context):
+        with db_api.CONTEXT_READER.using(context):
             mapped_router = self._map_display_name_or_tenant(router)
             mapped_router = self._map_state_and_status(mapped_router)
 
@@ -189,7 +185,7 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
         # set default router policies
         default_policy_dict = self._get_tenant_default_router_policy(tenant_id)
 
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             mapped_router = self._map_display_name_or_tenant(router)
             mapped_router = self._map_state_and_status(mapped_router)
             # populate external tenant_id if it is absent for external network,
@@ -249,10 +245,11 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
 
         orig_router = super(L3RestProxy, self).get_router(context, router_id)
         tenant_id = orig_router["tenant_id"]
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             setattr(context, 'GUARD_TRANSACTION', False)
             new_router = super(L3RestProxy,
                                self).update_router(context, router_id, router)
+
             router = self._update_ext_gateway_info(context, new_router)
 
             # update router on network controller
@@ -277,7 +274,7 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
         tenant_id = orig_router['tenant_id']
 
         # remove tenant policies if this was the last router under tenant
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             upstream_routers = super(L3RestProxy, self).get_routers(
                 context, filters={"tenant_id": [tenant_id]})
 
@@ -290,7 +287,7 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
     @add_debug_log
     @log_helper.log_method_call
     def delete_router(self, context, router_id):
-        with db.context_manager.reader.using(context):
+        with db_api.CONTEXT_READER.using(context):
             # Ensure that the router is not used
             router_filter = {'router_id': [router_id]}
             fips = self.get_floatingips_count(context.elevated(),
@@ -308,40 +305,40 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
 
         super(L3RestProxy, self).delete_router(context, router_id)
 
-    @log_helper.log_method_call
-    def router_interface_before_create_callback(self, resource, event, trigger,
-                                                **kwargs):
-        context = kwargs.get('context')
-        router = kwargs.get('router_db')
-        port = kwargs.get('port')
-        interface_info = kwargs.get('interface_info')
-        router_id = kwargs.get('router_id')
-
-        if 'port_id' in interface_info:
-            subnet_id = port['fixed_ips'][0]['subnet_id']
-        elif 'subnet_id' in interface_info:
-            subnet_id = interface_info['subnet_id']
-        else:
-            msg = _("Either subnet_id or port_id must be specified")
-            raise exceptions.BadRequest(resource='router', msg=msg)
-
-        # bookmark for delete in case of transaction rollback
-        self.txn_cache.add_transaction(interface_info[BSN_TRANSACTION_ID],
-                                       subnet_id)
-        with db.context_manager.reader.using(context):
-            # we will use the port's subnet id as interface's id
-            intf_details = self._get_router_intf_details(context, port,
-                                                         subnet_id)
-
-            # create interface on the network controller
-            self.servers.rest_add_router_interface(
-                router['tenant_id'], router_id, intf_details)
-
     @log_helper.log_method_call
     def router_interface_after_create_callback(self, resource, event, trigger,
                                                **kwargs):
         context = kwargs.get('context')
         port = kwargs.get('port')
+        router_id = kwargs.get('router_id')
+        interface_info = kwargs.get('interface_info')
+
+        subnet_id_list = []
+
+        if 'port_id' in interface_info:
+            for fixed_ip in port['fixed_ips']:
+                subnet_id_list.append(fixed_ip['subnet_id'])
+
+        elif 'subnet_id' in interface_info:
+            subnet_id_list.append(interface_info['subnet_id'])
+        else:
+            msg = _("Either subnet_id or port_id must be specified")
+            raise exceptions.BadRequest(resource='router', msg=msg)
+
+        for subnet_id in subnet_id_list:
+            # bookmark for delete in case of transaction rollback
+            self.txn_cache.add_transaction(interface_info[BSN_TRANSACTION_ID],
+                                           subnet_id)
+            with db_api.CONTEXT_READER.using(context):
+                # we will use the port's subnet id as interface's id
+                intf_details = self._get_router_intf_details(context, port,
+                                                             subnet_id)
+
+                tenant_id = intf_details['subnet']['project_id']
+
+                # create interface on the network controller
+                self.servers.rest_add_router_interface(tenant_id, router_id,
+                                                       intf_details)
 
         directory.get_plugin().update_port(context, port['id'],
                                            {'port': {'status': 'ACTIVE'}})
@@ -378,21 +375,24 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
         router = self._get_router(context, router_id)
         tenant_id = router['tenant_id']
 
+        subnet_id_list = []
+
         # we will first get the interface identifier before deleting in the DB
         if not interface_info:
             msg = _("Either subnet_id or port_id must be specified")
             raise exceptions.BadRequest(resource='router', msg=msg)
         if 'port_id' in interface_info:
             port = self._get_port(context, interface_info['port_id'])
-            interface_id = port['fixed_ips'][0]['subnet_id']
+            for fixed_ip in port['fixed_ips']:
+                subnet_id_list.append(fixed_ip['subnet_id'])
         elif 'subnet_id' in interface_info:
             subnet = self._get_subnet(context, interface_info['subnet_id'])
-            interface_id = subnet['id']
+            subnet_id_list.append(subnet['id'])
         else:
             msg = _("Either subnet_id or port_id must be specified")
             raise exceptions.BadRequest(resource='router', msg=msg)
 
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             # remove router in DB
             # TODO(wolverineav): hack until fixed at right place
             setattr(context, 'GUARD_TRANSACTION', False)
@@ -401,10 +401,11 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
                                                           router_id,
                                                           interface_info)
 
-            # create router on the network controller
-            self.servers.rest_remove_router_interface(tenant_id, router_id,
-                                                      interface_id)
-            return del_ret
+            for subnet_id in subnet_id_list:
+                # remove all related router interfaces
+                self.servers.rest_remove_router_interface(tenant_id, router_id,
+                                                          subnet_id)
+        return del_ret
 
     # add floating_port_id into the dict for later port mac lookup
     def _make_floatingip_dict(self, floatingip, fields=None,
@@ -413,12 +414,12 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
             floatingip, fields=fields,
             process_extensions=process_extensions)
         res['floating_port_id'] = floatingip['floating_port_id']
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
     @add_debug_log
     @log_helper.log_method_call
     def create_floatingip(self, context, floatingip):
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             # create floatingip in DB
             # TODO(wolverineav): hack until fixed at right place
             setattr(context, 'GUARD_TRANSACTION', False)
@@ -448,7 +449,7 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
     @add_debug_log
     @log_helper.log_method_call
     def update_floatingip(self, context, id, floatingip):
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             # update floatingip in DB
             # TODO(wolverineav): hack until fixed at right place
             setattr(context, 'GUARD_TRANSACTION', False)
@@ -472,7 +473,7 @@ class L3RestProxy(cplugin.NeutronRestProxyV2Base,
     @add_debug_log
     @log_helper.log_method_call
     def delete_floatingip(self, context, id):
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             # delete floating IP in DB
             # TODO(wolverineav): hack until fixed at right place
             setattr(context, 'GUARD_TRANSACTION', False)
