@@ -42,7 +42,6 @@ on port-attach) on an additional PUT to do a bulk dump of all persistent data.
 
 import copy
 import functools
-import httplib
 import re
 
 import eventlet
@@ -50,6 +49,7 @@ from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
 from oslo_utils import importutils
+from six.moves import http_client
 from sqlalchemy.orm import exc as sqlexc
 
 from neutron.agent import rpc as agent_rpc
@@ -59,12 +59,9 @@ from neutron.api.rpc.agentnotifiers import dhcp_rpc_agent_api
 from neutron.api.rpc.handlers import dhcp_rpc
 from neutron.api.rpc.handlers import metadata_rpc
 from neutron.api.rpc.handlers import securitygroups_rpc
-from neutron.common import rpc as n_rpc
-from neutron.common import topics
 from neutron.db import agents_db
 from neutron.db import agentschedulers_db
 from neutron.db import allowedaddresspairs_db as addr_pair_db
-from neutron.db import api as db
 from neutron.db import db_base_plugin_v2
 from neutron.db import external_net_db
 from neutron.db import extradhcpopt_db
@@ -73,6 +70,7 @@ from neutron.db.models import securitygroup as sg_db
 from neutron.db import models_v2
 from neutron.db import securitygroups_rpc_base as sg_db_rpc
 
+from neutron_lib.agent import topics
 from neutron_lib.api.definitions import allowedaddresspairs as addr_pair
 from neutron_lib.api.definitions import external_net as extnet_apidef
 from neutron_lib.api.definitions import extra_dhcp_opt as edo_ext
@@ -80,9 +78,12 @@ from neutron_lib.api.definitions import l3 as l3_apidef
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as const
 from neutron_lib import context as qcontext
+from neutron_lib.db import api as db_api
+from neutron_lib.db import utils as db_utils
 from neutron_lib import exceptions as lib_exceptions
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
+from neutron_lib import rpc as n_rpc
 from neutron_lib.utils import runtime
 
 from networking_bigswitch.plugins.bigswitch import config as pl_config
@@ -101,7 +102,7 @@ SYNTAX_ERROR_MESSAGE = _('Syntax error in server config file, aborting plugin')
 METADATA_SERVER_IP = '169.254.169.254'
 
 
-class AgentNotifierApi(sg_rpc.SecurityGroupAgentRpcApiMixin):
+class AgentNotifierApi(securitygroups_rpc.SecurityGroupAgentRpcApiMixin):
 
     def __init__(self, topic):
         self.topic = topic
@@ -130,7 +131,7 @@ class SecurityGroupServerRpcMixin(sg_db_rpc.SecurityGroupServerRpcMixin):
         LOG.debug("get_port_and_sgs() called for port_id %s", port_id)
         sg_binding_port = sg_db.SecurityGroupPortBinding.port_id
 
-        with db.context_manager.reader.using(context):
+        with db_api.CONTEXT_READER.using(context):
             query = context.session.query(
                 models_v2.Port,
                 sg_db.SecurityGroupPortBinding.security_group_id
@@ -166,7 +167,7 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         self._setup_rpc()
 
     def _setup_rpc(self):
-        self.conn = n_rpc.create_connection()
+        self.conn = n_rpc.Connection()
         self.topic = topics.PLUGIN
         self.notifier = AgentNotifierApi(topics.AGENT)
         # init dhcp agent support
@@ -232,7 +233,7 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         return self._get_all_data(
             get_floating_ips=self.l3_bsn_plugin,
             get_routers=self.l3_bsn_plugin,
-            get_sgs=True)
+            get_sgs=cfg.CONF.RESTPROXY.sync_security_groups)
 
     def _get_all_data(self, get_ports=True, get_floating_ips=True,
                       get_routers=True, get_sgs=True):
@@ -265,10 +266,11 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
                         new_subnets.append(subnet)
                     mapped_network['subnets'] = new_subnets
 
-                flips_n_ports = mapped_network
                 if get_floating_ips:
                     flips_n_ports = self._get_network_with_floatingips(
                         mapped_network)
+                else:
+                    flips_n_ports = mapped_network
 
                 if get_ports:
                     ports = []
@@ -299,7 +301,13 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
                             'id': port.get('device_id'),
                             'mac': port.get('mac_address'),
                         }
+
+                        if not get_sgs:
+                            if 'security_groups' in mapped_port:
+                                del mapped_port['security_groups']
+
                         ports.append(mapped_port)
+
                     flips_n_ports['ports'] = ports
 
                 if flips_n_ports:
@@ -347,11 +355,13 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
                     router_ports = self.get_ports(admin_context,
                                                   filters=router_filter) or []
                     for port in router_ports:
-                        subnet_id = port['fixed_ips'][0]['subnet_id']
-                        intf_details = self._get_router_intf_details(
-                            admin_context, port, subnet_id)
 
-                        interfaces.append(intf_details)
+                        for fixed_ip in port['fixed_ips']:
+                            subnet_id = fixed_ip['subnet_id']
+                            intf_details = self._get_router_intf_details(
+                                admin_context, port, subnet_id)
+
+                            interfaces.append(intf_details)
 
                     mapped_router['interfaces'] = interfaces
 
@@ -370,8 +380,7 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
             # L3 plugin also includes tenant policies
             # data.update({'policies': tenant_policies})
 
-        if (get_sgs and self.l3_plugin
-                and cfg.CONF.RESTPROXY.sync_security_groups):
+        if get_sgs and self.l3_plugin:
             sgs = plugin.get_security_groups(admin_context) or []
             new_sgs = []
             for sg in sgs:
@@ -483,7 +492,7 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         if context is None:
             context = qcontext.get_admin_context()
         # start a sub-transaction to avoid breaking parent transactions
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             subnets = self._get_subnets_by_network(context,
                                                    net_id)
         subnets_details = []
@@ -633,8 +642,13 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         # display-name is also mapped here
         mapped_network = self._get_mapped_network_with_subnets(network,
                                                                context)
-        net_fl_ips = self._get_network_with_floatingips(mapped_network,
-                                                        context)
+
+        if self.l3_bsn_plugin:
+            net_fl_ips = self._get_network_with_floatingips(mapped_network,
+                                                            context)
+        else:
+            net_fl_ips = mapped_network
+
         if not tenant_id:
             tenant_id = servermanager.SERVICE_TENANT
             net_fl_ips['tenant_id'] = servermanager.SERVICE_TENANT
@@ -826,8 +840,8 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         # if bridge_name not available, sets it to just 'host-id'
         vif_type = prepped_port.get(portbindings.VIF_TYPE)
         if (vif_type and
-                (vif_type == portbindings.VIF_TYPE_OVS
-                 or vif_type == portbindings.VIF_TYPE_VHOST_USER)):
+                (vif_type == portbindings.VIF_TYPE_OVS or
+                 vif_type == portbindings.VIF_TYPE_VHOST_USER)):
             prepped_port[portbindings.HOST_ID] = (
                 self._get_ovs_dpdk_port_hostid(prepped_port, network))
 
@@ -846,7 +860,14 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
                             resource['status'])
 
     def _get_router_intf_details(self, context, port, subnet_id):
+        """This gets router interface for 1 single subnet
 
+        For multiple subnets, this should be called multiple times
+        :param context:
+        :param port:
+        :param subnet_id:
+        :return:
+        """
         # we will use the network id as interface's id
         subnet = self.get_subnet(context, subnet_id)
         net_id = subnet['network_id']
@@ -862,8 +883,12 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
         }
 
         # get gateway_ip from port instead of gateway_ip
+        # every port can have multiple fixed_ips
         if port.get("fixed_ips"):
-            data['ip_address'] = port["fixed_ips"][0]['ip_address']
+            for fixed_ip in port["fixed_ips"]:
+                if fixed_ip['subnet_id'] == subnet_id:
+                    data['ip_address'] = fixed_ip['ip_address']
+                    break
 
         return data
 
@@ -928,7 +953,7 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
             # and the data in the backend.
             # Run a sync to get it consistent.
             if (cfg.CONF.RESTPROXY.auto_sync_on_failure and
-                    e.status == httplib.NOT_FOUND and
+                    e.status == http_client.NOT_FOUND and
                     servermanager.NXNETWORK in e.reason):
                 LOG.error("Inconsistency with backend controller "
                           "triggering full synchronization.")
@@ -967,7 +992,7 @@ class NeutronRestProxyV2Base(db_base_plugin_v2.NeutronDbPluginV2,
     # NOTE(kevinbenton): workaround for eventlet/mysql deadlock
     @runtime.synchronized('bsn-port-barrier')
     def _set_port_status(self, port_id, status):
-        session = db.get_writer_session()
+        session = db_api.get_writer_session()
         try:
             port = session.query(models_v2.Port).filter_by(id=port_id).one()
             port['status'] = status
@@ -1017,11 +1042,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
 
         # init network ctrl connections
         self.servers = servermanager.ServerPool()
-        self.servers.get_topo_function = self._get_all_data
-        self.servers.get_topo_function_args = {'get_ports': True,
-                                               'get_floating_ips': True,
-                                               'get_routers': True,
-                                               'get_sgs': True}
+        self.servers.get_topo_function = self._get_all_data_auto
 
         self.network_scheduler = importutils.import_object(
             cfg.CONF.network_scheduler_driver
@@ -1036,7 +1057,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
         self.add_periodic_dhcp_agent_status_check()
         LOG.debug("NeutronRestProxyV2: initialization done")
 
-    @db.context_manager.writer
+    @db_api.CONTEXT_WRITER
     @add_debug_log
     def create_network(self, context, network):
         """Create a network.
@@ -1078,7 +1099,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
         # return created network
         return new_net
 
-    @db.context_manager.writer
+    @db_api.CONTEXT_WRITER
     @add_debug_log
     def update_network(self, context, net_id, network):
         """Updates the properties of a particular Virtual Network.
@@ -1113,7 +1134,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
         self._send_update_network(new_net, context)
         return new_net
 
-    @db.context_manager.writer
+    @db_api.CONTEXT_WRITER
     @add_debug_log
     def delete_network(self, context, net_id):
         """Delete a network.
@@ -1166,7 +1187,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
         :raises: RemoteRestError
         """
         # Update DB in new session so exceptions rollback changes
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             self._ensure_default_security_group_on_port(context, port)
             sgids = self._get_security_groups_on_port(context, port)
             # non-router port status is set to pending. it is then updated
@@ -1179,8 +1200,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
             new_port = super(NeutronRestProxyV2, self).create_port(context,
                                                                    port)
             self._process_port_create_security_group(context, new_port, sgids)
-        if (portbindings.HOST_ID in port['port']
-                and 'id' in new_port):
+        if (portbindings.HOST_ID in port['port'] and 'id' in new_port):
             host_id = port['port'][portbindings.HOST_ID]
             porttracker_db.put_port_hostid(context, new_port['id'],
                                            host_id)
@@ -1217,19 +1237,19 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
         return new_port
 
     def get_port(self, context, id, fields=None):
-        with db.context_manager.reader.using(context):
+        with db_api.CONTEXT_READER.using(context):
             port = super(NeutronRestProxyV2, self).get_port(context, id,
                                                             fields)
             self._extend_port_dict_binding(context, port)
-        return self._fields(port, fields)
+        return db_utils.resource_fields(port, fields)
 
     def get_ports(self, context, filters=None, fields=None):
-        with db.context_manager.reader.using(context):
+        with db_api.CONTEXT_READER.using(context):
             ports = super(NeutronRestProxyV2, self).get_ports(context, filters,
                                                               fields)
             for port in ports:
                 self._extend_port_dict_binding(context, port)
-        return [self._fields(port, fields) for port in ports]
+        return [db_utils.resource_fields(port, fields) for port in ports]
 
     @add_debug_log
     def update_port(self, context, port_id, port):
@@ -1263,7 +1283,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
 
         # Validate Args
         orig_port = super(NeutronRestProxyV2, self).get_port(context, port_id)
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             # Update DB
             new_port = super(NeutronRestProxyV2,
                              self).update_port(context, port_id, port)
@@ -1276,8 +1296,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
                                                  new_port)
             old_host_id = porttracker_db.get_port_hostid(context,
                                                          orig_port['id'])
-            if (portbindings.HOST_ID in port['port']
-                    and 'id' in new_port):
+            if (portbindings.HOST_ID in port['port'] and 'id' in new_port):
                 host_id = port['port'][portbindings.HOST_ID]
                 porttracker_db.put_port_hostid(context, new_port['id'],
                                                host_id)
@@ -1330,7 +1349,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
         # and l3-router.  If so, we should prevent deletion.
         if l3_port_check and self.l3_plugin:
             self.l3_plugin.prevent_l3_port_deletion(context, port_id)
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             if self.l3_plugin:
                 router_ids = self.l3_plugin.disassociate_floatingips(
                     context, port_id, do_notify=False)
@@ -1364,7 +1383,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
     def update_subnet(self, context, id, subnet):
         self._warn_on_state_status(subnet['subnet'])
 
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             # update subnet in DB
             new_subnet = super(NeutronRestProxyV2,
                                self).update_subnet(context, id, subnet)
@@ -1379,7 +1398,7 @@ class NeutronRestProxyV2(NeutronRestProxyV2Base,
     def delete_subnet(self, context, id):
         orig_subnet = super(NeutronRestProxyV2, self).get_subnet(context, id)
         net_id = orig_subnet['network_id']
-        with db.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             # delete subnet in DB
             super(NeutronRestProxyV2, self).delete_subnet(context, id)
             orig_net = super(NeutronRestProxyV2, self).get_network(context,
