@@ -1,3 +1,4 @@
+
 # Copyright 2014 Big Switch Networks, Inc.
 # All Rights Reserved.
 #
@@ -27,32 +28,33 @@ The following functionality is handled by this module:
 
 """
 import base64
-import httplib
+import os
 import re
 import socket
 import ssl
 import time
 
-from neutron_lib import exceptions
-
-from oslo_log import log as logging
-
 import eventlet
 import eventlet.corolocal
 from keystoneauth1.identity import v3
 from keystoneauth1 import session
 from keystoneclient.v3 import client as ksclient
+from neutron_lib import exceptions
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_serialization import jsonutils
+from oslo_utils import excutils
+import six
+from six.moves import http_client
+from sqlalchemy.types import Enum
+
 from networking_bigswitch.plugins.bigswitch.db import consistency_db as cdb
 from networking_bigswitch.plugins.bigswitch.i18n import _
 from networking_bigswitch.plugins.bigswitch.i18n import _LE
 from networking_bigswitch.plugins.bigswitch.i18n import _LI
 from networking_bigswitch.plugins.bigswitch.i18n import _LW
+from networking_bigswitch.plugins.bigswitch import state_syncer
 from networking_bigswitch.plugins.bigswitch.utils import Util
-import os
-from oslo_config import cfg
-from oslo_serialization import jsonutils
-from oslo_utils import excutils
-from sqlalchemy.types import Enum
 
 LOG = logging.getLogger(__name__)
 
@@ -81,6 +83,8 @@ TESTPATH_PATH = ('/testpath/controller-view'
                  '&dst-ip=%(dst-ip)s')
 TENANTPOLICY_RESOURCE_PATH = "/tenants/%s/policies"
 TENANTPOLICIES_PATH = "/tenants/%s/policies/%s"
+OOSM_OSP_RESOURCE_PATH = "/orchestrator/openstack"
+OOSM_OSP_LAST_UPDATE_PATH = OOSM_OSP_RESOURCE_PATH + "/%s/last-update"
 SUCCESS_CODES = range(200, 207)
 FAILURE_CODES = [0, 301, 302, 303, 400, 401, 403, 404, 500, 501, 502, 503,
                  504, 505]
@@ -90,6 +94,8 @@ HASH_MATCH_HEADER = 'X-BSN-BVS-HASH-MATCH'
 SERVICE_TENANT = 'VRRP_Service'
 KS_AUTH_GROUP_NAME = 'keystone_authtoken'
 KS_AUTH_DOMAIN_DEFAULT = 'default'
+# nova auth group name
+NOVA_AUTH_GROUP_NAME = 'nova'
 # error messages
 NXNETWORK = 'NXVNS'
 HTTP_SERVICE_UNAVAILABLE_RETRY_COUNT = 3
@@ -97,7 +103,7 @@ HTTP_SERVICE_UNAVAILABLE_RETRY_INTERVAL = 3
 
 KEYSTONE_SYNC_RATE_LIMIT = 30  # Limit KeyStone sync to once in 30 secs
 # TOPO_SYNC Responses
-TOPO_RESPONSE_OK = (httplib.OK, httplib.OK, True, True)
+TOPO_RESPONSE_OK = (http_client.OK, http_client.OK, True, True)
 TOPO_RESPONSE_FAIL = (0, None, None, None)
 
 # RE pattern for checking BCF supported names
@@ -233,9 +239,15 @@ def get_keystoneauth_cfg(conf, name):
     :param name: property name to be retrieved
     """
     try:
+        # expected response is a tuple consisting of
+        # (value_list, source_of_value). since keystone attributes are fields
+        # like username, password and domain name - they're not actually lists.
+        # so we pick the first item from the list
+        value_list, source_of_value = conf._namespace._get_file_value(
+            [(KS_AUTH_GROUP_NAME, name)])
 
-        value_list = conf._namespace._get_file_value([(KS_AUTH_GROUP_NAME,
-                                                       name)])
+        LOG.debug('KEYSTONE_AUTHTOKEN value for %(name)s is %(value)s',
+                  {'name': name, 'value': value_list[0]})
         return value_list[0]
     except KeyError as e:
         LOG.warning("Config does not have property %(name)s "
@@ -243,14 +255,41 @@ def get_keystoneauth_cfg(conf, name):
         raise e
 
 
+def get_novaauth_cfg(conf, name):
+    """get the nova auth cfg
+
+    Fetch value of nova group from config file when not
+    available as part of GroupAttr.
+
+    :rtype: String
+    :param conf: oslo config cfg.CONF
+    :param name: property name to be retrieved
+    """
+    try:
+
+        value_list = conf._namespace._get_file_value([(NOVA_AUTH_GROUP_NAME,
+                                                       name)])
+        if isinstance(value_list[0], list):
+            # api change in train, needs more investigation
+            return value_list[0][0]
+        else:
+            return value_list[0]
+    except KeyError:
+        LOG.debug("Config does not have property %(name)s "
+                  "in group nova", {'name': name})
+        # no need to raise exception - if property is missing in nova group,
+        # we reuse what was set for keystone :)
+        # raise e
+
+
 class ServerProxy(object):
     """REST server proxy to a network controller."""
 
-    def __init__(self, server, port, ssl, auth, neutron_id, timeout,
+    def __init__(self, server, port, is_ssl_enabled, auth, neutron_id, timeout,
                  base_uri, name, mypool, combined_cert):
         self.server = server
         self.port = port
-        self.ssl = ssl
+        self.is_ssl_enabled = is_ssl_enabled
         self.base_uri = base_uri
         self.timeout = timeout
         self.name = name
@@ -262,16 +301,40 @@ class ServerProxy(object):
         self.capabilities = []
         # enable server to reference parent pool
         self.mypool = mypool
-        # cache connection here to avoid a SSL handshake for every connection
-        self.currentconn = None
+
+        # TODO(weifan): cache connection to avoid a SSL handshake for every
+        # connection
+        self.cached_conn = None
+
+        # ssl_context is the new standard interface for handling certificates
+        # initialize ssl context
+        self.ssl_context = None
 
         if auth:
             if ':' in auth:
-                self.auth = 'Basic ' + base64.encodestring(auth).strip()
+                if six.PY2:
+                    self.auth = 'Basic ' + base64.encodestring(auth).strip()
+                else:
+                    self.auth = 'Basic ' + base64.b64encode(auth.encode(
+                        'utf-8')).decode('utf-8')
             else:
                 self.auth_token = 'session_cookie="' + auth + '"'
 
-        self.combined_cert = combined_cert
+        if is_ssl_enabled:
+            # initialize ssl context
+            self.ssl_context = None
+
+            # Uses PROTOCOL_TLS (new default)
+            # or PROTOCOL_SSLv23(deprecated, old default)
+            # both works with BCF TLS connections
+            self.ssl_context = ssl.create_default_context(cafile=combined_cert)
+
+            if six.PY2:
+                # Monkey Patch to skip hostname check for ssl in py2
+                # There is a bug with the old version
+                ssl.match_hostname = lambda cert, hostname: True
+            else:
+                self.ssl_context.check_hostname = False
 
     def get_capabilities(self):
         try:
@@ -311,8 +374,9 @@ class ServerProxy(object):
             headers['Authorization'] = self.auth
 
         LOG.debug("ServerProxy: server=%(server)s, port=%(port)d, "
-                  "ssl=%(ssl)r",
-                  {'server': self.server, 'port': self.port, 'ssl': self.ssl})
+                  "is_ssl_enabled=%(is_ssl_enabled)r",
+                  {'server': self.server, 'port': self.port,
+                   'is_ssl_enabled': self.is_ssl_enabled})
         LOG.debug("ServerProxy: resource=%(resource)s, data=%(data)r, "
                   "headers=%(headers)r, action=%(action)s",
                   {'resource': resource, 'data': data, 'headers': headers,
@@ -327,26 +391,39 @@ class ServerProxy(object):
             # need a new connection if timeout has changed
             reconnect = True
 
-        if not self.currentconn or reconnect:
-            if self.currentconn:
-                self.currentconn.close()
-            if self.ssl:
-                currentconn = HTTPSConnectionWithValidation(
-                    self.server, self.port, timeout=timeout)
+        if not self.cached_conn or reconnect:
+            if self.cached_conn:
+                self.cached_conn.close()
+            if self.is_ssl_enabled:
+                currentconn = http_client.HTTPSConnection(
+                    self.server, self.port, timeout=timeout,
+                    context=self.ssl_context)
+
                 if currentconn is None:
                     LOG.error('ServerProxy: Could not establish HTTPS '
                               'connection')
                     return 0, None, None, None
-                currentconn.combined_cert = self.combined_cert
+
+                # TODO(weifan): cache connection and reuse
+                # require keep-alive
+                # self.cached_conn = currentconn
+
             else:
-                currentconn = httplib.HTTPConnection(
+                currentconn = http_client.HTTPConnection(
                     self.server, self.port, timeout=timeout)
                 if currentconn is None:
                     LOG.error('ServerProxy: Could not establish HTTP '
                               'connection')
                     return 0, None, None, None
 
+                # TODO(weifan): cache connection and reuse
+                # require keep-alive
+                # self.cached_conn = currentconn
+
         try:
+            # TODO(weifan): use cache connection instead of currentconn
+            # require keep alive
+
             bcf_request_time = time.time()
             currentconn.request(action, uri, body, headers)
             response = currentconn.getresponse()
@@ -363,7 +440,12 @@ class ServerProxy(object):
                     pass
 
             ret = (response.status, response.reason, respstr, respdata)
-        except httplib.HTTPException:
+
+            # TODO(weifan): use cache connection
+            # currently closing every connection after use
+            currentconn.close()
+
+        except http_client.HTTPException:
             # If we were using a cached connection, try again with a new one.
             with excutils.save_and_reraise_exception() as ctxt:
                 currentconn.close()
@@ -387,6 +469,7 @@ class ServerProxy(object):
                                                  'reason': ret[1],
                                                  'ret': ret[2],
                                                  'data': ret[3]})
+
         return ret
 
 
@@ -409,12 +492,15 @@ class ServerPool(object):
         # till next failure). Use 'server_auth' to encode api-key
         servers = cfg.CONF.RESTPROXY.servers
         self.auth = cfg.CONF.RESTPROXY.server_auth
-        self.ssl = cfg.CONF.RESTPROXY.server_ssl
+        self.is_ssl_enabled = cfg.CONF.RESTPROXY.server_ssl
         self.neutron_id = cfg.CONF.RESTPROXY.neutron_id
         # unicode config
         self.cfg_unicode_enabled = cfg.CONF.RESTPROXY.naming_scheme_unicode
         self.capabilities = set()
 
+        # cluster state sync interval
+        self.state_sync_interval = cfg.CONF.RESTPROXY.state_sync_interval
+
         if 'keystone_authtoken' in cfg.CONF:
             self.auth_user = get_keystoneauth_cfg(cfg.CONF, 'username')
             self.auth_password = get_keystoneauth_cfg(cfg.CONF, 'password')
@@ -442,6 +528,43 @@ class ServerPool(object):
         elif "v3" not in self.auth_url:
             self.auth_url = "%s/v3" % self.auth_url
 
+        # load nova auth separately if available (true for Devstack)
+        nova_auth_dict = {}
+        if 'nova' in cfg.CONF:
+            nova_auth_dict['auth_url'] = self.auth_url
+            username = get_novaauth_cfg(cfg.CONF, 'username')
+            nova_auth_dict['username'] = (self.auth_user
+                                          if username is None else username)
+            password = get_novaauth_cfg(cfg.CONF, 'password')
+            nova_auth_dict['password'] = (self.auth_password
+                                          if password is None else password)
+            project_name = get_novaauth_cfg(cfg.CONF, 'project_name')
+            nova_auth_dict['project_name'] = (
+                self.auth_tenant
+                if project_name is None else project_name)
+            user_domain_name = get_novaauth_cfg(cfg.CONF, 'user_domain_name')
+            nova_auth_dict['user_domain_name'] = (
+                self.user_domain_name
+                if user_domain_name is None else user_domain_name)
+            project_domain_name = get_novaauth_cfg(
+                cfg.CONF, 'project_domain_name')
+            nova_auth_dict['project_domain_name'] = (
+                self.project_domain_name
+                if project_domain_name is None else project_domain_name)
+
+        # create a dict for keystone and nova auth to pass to state_syncer
+        ks_auth_dict = {}
+        ks_auth_dict['auth_url'] = self.auth_url
+        ks_auth_dict['username'] = self.auth_user
+        ks_auth_dict['password'] = self.auth_password
+        ks_auth_dict['project_name'] = self.auth_tenant
+        ks_auth_dict['user_domain_name'] = self.user_domain_name
+        ks_auth_dict['project_domain_name'] = self.project_domain_name
+
+        # once all auth info is loaded, init the state_syncer
+        self.state_syncer = state_syncer.StateSyncer(
+            ks_auth_dict=ks_auth_dict, nova_auth_dict=nova_auth_dict)
+
         self.base_uri = base_uri
         self.name = name
         # Cache for Openstack projects
@@ -458,7 +581,6 @@ class ServerPool(object):
         # Function to use to retrieve topology for consistency syncs.
         # Needs to be set by module that uses the servermanager.
         self.get_topo_function = None
-        self.get_topo_function_args = {}
 
         if not servers:
             raise cfg.Error(_('Servers not defined. Aborting server manager.'))
@@ -502,6 +624,21 @@ class ServerPool(object):
             self._keystone_sync,
             cfg.CONF.RESTPROXY.keystone_sync_interval)
 
+        # start a periodic state syncer with BCF
+        # this provides hypervisor, vm and other misc info to BCF for
+        # integration dashboard
+        if self.state_sync_interval > 0:
+            if 'orchestrator-state-sync' in self.get_capabilities():
+                eventlet.spawn_after(60,
+                                     self.state_syncer.periodic_update,
+                                     self.state_sync_interval)
+                LOG.info('Orchestrator State Sync is enabled.')
+            else:
+                LOG.info('BCF does not support orchestrator-state-sync, '
+                         'disabled it.')
+        else:
+            LOG.info('Orchestrator State Sync is disabled.')
+
     def get_capabilities(self):
         """Get capabilities
 
@@ -587,9 +724,12 @@ class ServerPool(object):
             return False
 
     def server_proxy_for(self, server, port):
+        # TODO(weifan): use ssl_context to load certificates instead of our
+        # own codes
         combined_cert = self._get_combined_cert_for_server(server, port)
-        return ServerProxy(server, port, self.ssl, self.auth, self.neutron_id,
-                           self.timeout, self.base_uri, self.name, mypool=self,
+        return ServerProxy(server, port, self.is_ssl_enabled, self.auth,
+                           self.neutron_id, self.timeout, self.base_uri,
+                           self.name, mypool=self,
                            combined_cert=combined_cert)
 
     def _get_combined_cert_for_server(self, server, port):
@@ -597,7 +737,7 @@ class ServerPool(object):
         # so we make one containing the trusted CAs and the corresponding
         # host cert for this server
         combined_cert = None
-        if self.ssl and not cfg.CONF.RESTPROXY.no_ssl_validation:
+        if self.is_ssl_enabled and not cfg.CONF.RESTPROXY.no_ssl_validation:
             base_ssl = cfg.CONF.RESTPROXY.ssl_cert_directory
             host_dir = os.path.join(base_ssl, 'host_certs')
             ca_dir = os.path.join(base_ssl, 'ca_certs')
@@ -659,9 +799,10 @@ class ServerPool(object):
         Grabs a certificate from a server and writes it to
         a given path.
         """
+
         try:
-            cert = ssl.get_server_certificate((server, port),
-                                              ssl_version=ssl.PROTOCOL_SSLv23)
+            cert = ssl.get_server_certificate((server, port))
+
         except Exception as e:
             raise cfg.Error(_('Could not retrieve initial '
                               'certificate from controller %(server)s. '
@@ -670,6 +811,7 @@ class ServerPool(object):
 
         LOG.warning("Storing to certificate for host %(server)s "
                     "at %(path)s", {'server': server, 'path': path})
+
         self._file_put_contents(path, cert)
 
         return cert
@@ -712,7 +854,7 @@ class ServerPool(object):
                 ret = active_server.rest_call(action, resource, data, headers,
                                               timeout,
                                               reconnect=self.always_reconnect)
-                if ret[0] != httplib.SERVICE_UNAVAILABLE:
+                if ret[0] != http_client.SERVICE_UNAVAILABLE:
                     break
                 eventlet.sleep(HTTP_SERVICE_UNAVAILABLE_RETRY_INTERVAL)
 
@@ -924,6 +1066,13 @@ class ServerPool(object):
 
     def rest_create_port(self, tenant_id, net_id, port):
         resource = ATTACHMENT_PATH % (tenant_id, net_id, port["id"])
+
+        # security-group info might be useful for some higher level
+        # operations, so we modify the json right before sending
+        if not cfg.CONF.RESTPROXY.sync_security_groups:
+            if 'security_groups' in port:
+                del port['security_groups']
+
         data = {"port": port}
         device_id = port.get("device_id")
         if not port["mac_address"] or not device_id:
@@ -934,7 +1083,9 @@ class ServerPool(object):
             return
         data["attachment"] = {"id": device_id,
                               "mac": port["mac_address"]}
+
         errstr = _("Unable to create remote port: %s")
+
         self.rest_action('PUT', resource, data, errstr)
 
     def rest_delete_port(self, tenant_id, network_id, port_id):
@@ -1001,6 +1152,22 @@ class ServerPool(object):
         errstr = _("Unable to delete remote policy: %s")
         self.rest_action('DELETE', resource, errstr=errstr)
 
+    def rest_update_osp_cluster_info(self, data):
+        resource = OOSM_OSP_RESOURCE_PATH
+
+        # nsapi does not append origination for cluster info
+        data['origination'] = self.neutron_id
+
+        errstr = _("Unable to update cluster visibility info: %s")
+        self.rest_action('POST', resource, data, errstr)
+
+    def rest_get_osp_cluster_last_update(self):
+        resource = OOSM_OSP_LAST_UPDATE_PATH % self.neutron_id
+        errstr = _("Failed to get last update time for the cluster: %s")
+        resp = self.rest_action('GET', resource, errstr=errstr)
+
+        return resp[3].get('last-update')
+
     def _consistency_watchdog(self, polling_interval=60):
         if 'consistency' not in self.get_capabilities():
             LOG.warning("Backend server(s) do not support automated "
@@ -1072,8 +1239,7 @@ class ServerPool(object):
                       "%(start_ts)s",
                       {'request_ts': cdb.convert_ts_to_datetime(curr_ts),
                        'start_ts': cdb.convert_ts_to_datetime(time.time())})
-            data = self.get_topo_function(
-                **self.get_topo_function_args)
+            data = self.get_topo_function()
             if not data:
                 # when keystone sync fails, it fails silently with data = None
                 # that is wrong, we need to raise an exception
@@ -1151,27 +1317,3 @@ class ServerPool(object):
         while True:
             eventlet.sleep(polling_interval)
             self._update_tenant_cache()
-
-
-class HTTPSConnectionWithValidation(httplib.HTTPSConnection):
-
-    # If combined_cert is None, the connection will continue without
-    # any certificate validation.
-    combined_cert = None
-
-    def connect(self):
-        sock = socket.create_connection((self.host, self.port),
-                                        self.timeout, self.source_address)
-        if self._tunnel_host:
-            self.sock = sock
-            self._tunnel()
-
-        if self.combined_cert:
-            self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
-                                        cert_reqs=ssl.CERT_REQUIRED,
-                                        ca_certs=self.combined_cert,
-                                        ssl_version=ssl.PROTOCOL_SSLv23)
-        else:
-            self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
-                                        cert_reqs=ssl.CERT_NONE,
-                                        ssl_version=ssl.PROTOCOL_SSLv23)
