@@ -14,17 +14,10 @@
 #    under the License.
 import copy
 import datetime
-import httplib
 import os
 
 import eventlet
-from oslo_config import cfg
-from oslo_log import log
-import oslo_messaging
-from oslo_utils import excutils
-from oslo_utils import timeutils
 
-from neutron.common import rpc as n_rpc
 from neutron.extensions import securitygroup as ext_sg
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
@@ -34,19 +27,26 @@ from neutron_lib import constants as const
 from neutron_lib import context as ctx
 from neutron_lib.plugins import directory
 from neutron_lib.plugins.ml2 import api
+from neutron_lib import rpc as lib_rpc
+from oslo_config import cfg
+from oslo_log import log
+import oslo_messaging
+from oslo_utils import excutils
+from oslo_utils import timeutils
+from six.moves import http_client
 
 from networking_bigswitch.plugins.bigswitch import config as pl_config
+from networking_bigswitch.plugins.bigswitch import constants as bsn_consts
 from networking_bigswitch.plugins.bigswitch.i18n import _
 from networking_bigswitch.plugins.bigswitch.i18n import _LE
-from networking_bigswitch.plugins.bigswitch.i18n import _LI
 from networking_bigswitch.plugins.bigswitch.i18n import _LW
 from networking_bigswitch.plugins.bigswitch import plugin
 from networking_bigswitch.plugins.bigswitch import servermanager
+from networking_bigswitch.plugins.bigswitch.utils import Util
 
 EXTERNAL_PORT_OWNER = 'neutron:external_port'
 ROUTER_GATEWAY_PORT_OWNER = 'network:router_gateway'
-OVS_AGENT_INI_FILEPATH = '/etc/neutron/plugins/ml2/openvswitch_agent.ini'
-RH_NET_CONF_PATH = "/etc/os-net-config/config.json"
+
 LOG = log.getLogger(__name__)
 add_debug_log = plugin.add_debug_log
 
@@ -54,51 +54,6 @@ add_debug_log = plugin.add_debug_log
 CACHE_VSWITCH_TIME = 60
 
 
-def _read_ovs_bridge_mappings():
-    """Read the 'bridge_mappings' property from openvswitch_agent.ini
-
-    This is done for Redhat environments, to allow an improved
-    learning/programming of interface groups based on ports and the network
-    to which the ports belong to.
-
-    :return: bridge_mappings dictionary {'physnet_name': 'bridge_name', ...}
-                {} empty dictionary when not found
-    """
-    mapping = {}
-    mapping_str = None
-    # read openvswitch_agent.ini for bridge_mapping info
-    if not os.path.isfile(OVS_AGENT_INI_FILEPATH):
-        # if ovs_agent.ini doesn't exists, return empty mapping
-        LOG.warning(_LW("Unable to read OVS bridge_mappings, "
-                        "openvswitch_agent.ini file not present."))
-        return mapping
-
-    with open(OVS_AGENT_INI_FILEPATH) as f:
-        for line in f:
-            if ('#' not in line and
-                    ('=' in line and 'bridge_mappings' in line)):
-                # typical config line looks like the following:
-                # bridge_mappings = datacentre:br-ex,dpdk:br-link
-                key, value = line.split('=', 1)
-                mapping_str = value.strip()
-
-    # parse comma separated physnet list into individual mappings
-    if not mapping_str:
-        # if file did not have bridge_mappings, return empty mapping
-        LOG.warning(_LW(
-            "Unable to read OVS bridge_mappings, either the line is commented "
-            "or not present in openvswitch_agent.ini."))
-        return mapping
-
-    phy_map_list = mapping_str.split(',')
-    for phy_map in phy_map_list:
-        phy, bridge = phy_map.split(':')
-        mapping[phy.strip()] = bridge.strip()
-
-    LOG.info(_LI("OVS bridge_mappings are: %(br_map)s"), {'br_map': mapping})
-    return mapping
-
-
 class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
                                api.MechanismDriver):
 
@@ -118,11 +73,8 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
 
         # init network ctrl connections
         self.servers = servermanager.ServerPool()
-        self.servers.get_topo_function = self._get_all_data
-        self.servers.get_topo_function_args = {'get_ports': True,
-                                               'get_floating_ips': True,
-                                               'get_routers': True,
-                                               'get_sgs': True}
+        self.servers.get_topo_function = self._get_all_data_auto
+
         # perform one forced topo_sync after 60secs. delayed to let plugin
         # initialization complete
         eventlet.spawn_after(60, self.servers.force_topo_sync,
@@ -132,8 +84,8 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
         # if os-net-config is present, attempt to read physnet bridge_mappings
         # from openvswitch_agent.ini
         self.bridge_mappings = {}
-        if os.path.isfile(RH_NET_CONF_PATH):
-            self.bridge_mappings = _read_ovs_bridge_mappings()
+        if os.path.isfile(bsn_consts.RH_NET_CONF_PATH):
+            self.bridge_mappings = Util.read_ovs_bridge_mappings()
         # Track hosts running IVS to avoid excessive calls to the backend
         self.ivs_host_cache = {}
         self.setup_rpc_callbacks()
@@ -181,7 +133,7 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
         keystone_target = oslo_messaging.Target(
             topic='#', exchange='keystone', server=cfg.CONF.host)
         self.listener = oslo_messaging.get_notification_listener(
-            n_rpc.TRANSPORT, [target, keystone_target], [self],
+            lib_rpc.TRANSPORT, [target, keystone_target], [self],
             executor='eventlet', allow_requeue=False)
         self.listener.start()
 
@@ -391,7 +343,7 @@ class BigSwitchMechanismDriver(plugin.NeutronRestProxyV2Base,
             except servermanager.RemoteRestError as e:
                 with excutils.save_and_reraise_exception() as ctxt:
                     if (cfg.CONF.RESTPROXY.auto_sync_on_failure and
-                            e.status == httplib.NOT_FOUND and
+                            e.status == http_client.NOT_FOUND and
                             servermanager.NXNETWORK in e.reason):
                         ctxt.reraise = False
                         LOG.error("Inconsistency with backend controller "
