@@ -17,8 +17,6 @@
 import functools
 
 import mock
-from oslo_serialization import jsonutils
-
 from neutron.conf.plugins.ml2 import config as ml2_config
 from neutron.db import l3_db
 from neutron.plugins.ml2.drivers import type_vlan as vlan_config
@@ -28,30 +26,30 @@ from neutron.tests.unit.extensions import test_l3
 from neutron.tests.unit.extensions import test_securitygroup as test_sg
 from neutron.tests.unit.plugins.ml2 import test_plugin
 from neutron_lib.api.definitions import portbindings
-from neutron_lib import constants as lib_constants
 from neutron_lib import context as neutron_context
 from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
+from oslo_serialization import jsonutils
+from oslo_utils import uuidutils
 
 from networking_bigswitch.plugins.bigswitch import config as pl_config
 from networking_bigswitch.plugins.bigswitch import servermanager
 from networking_bigswitch.plugins.ml2.drivers.mech_bigswitch \
     import driver as bsn_driver
-# from networking_bigswitch.tests.unit.bigswitch.fake_server \
-#     import HTTPResponseMock
+from networking_bigswitch.tests.unit.bigswitch.fake_server \
+    import HTTPResponseMock
+from networking_bigswitch.tests.unit.bigswitch.mock_paths import DRIVER
+from networking_bigswitch.tests.unit.bigswitch.mock_paths import DRIVER_MOD
+from networking_bigswitch.tests.unit.bigswitch.mock_paths import HTTPCON
+from networking_bigswitch.tests.unit.bigswitch.mock_paths import SERVER_POOL
 import networking_bigswitch.tests.unit.bigswitch.test_restproxy_plugin as trp
-from oslo_utils import uuidutils
 
 _uuid = uuidutils.generate_uuid
 
 PHYS_NET = 'physnet1'
 VLAN_START = 1000
 VLAN_END = 1100
-SERVER_MANAGER = 'networking_bigswitch.plugins.bigswitch.servermanager'
-SERVER_POOL = SERVER_MANAGER + '.ServerPool'
-DRIVER_MOD = 'networking_bigswitch.plugins.ml2.drivers.mech_bigswitch.driver'
-DRIVER = DRIVER_MOD + '.BigSwitchMechanismDriver'
-HTTPCON = SERVER_MANAGER + '.httplib.HTTPConnection'
 
 
 class TestBigSwitchMechDriverBase(trp.BigSwitchProxyPluginV2TestCase):
@@ -79,34 +77,34 @@ class TestBigSwitchMechDriverBase(trp.BigSwitchProxyPluginV2TestCase):
 class TestBigSwitchMechDriverNetworksV2(test_db_base_plugin_v2.TestNetworksV2,
                                         TestBigSwitchMechDriverBase):
     def setUp(self, plugin=None, service_plugins=None, ext_mgr=None):
-        # topo sync can be triggered outside of watchdog
-        self.startTopoSyncPatch()
-
         TestBigSwitchMechDriverBase.setUp(self,
                                           plugin=plugin,
                                           service_plugins=service_plugins,
                                           ext_mgr=ext_mgr)
 
     def test_create_network(self):
-        # TODO(weifan): Figure out why topo sync mock does not work here
-        # Skip for now
-        pass
-        # name = 'net1'
-        # keys = [('subnets', []), ('name', name), ('admin_state_up', True),
-        #         ('status', self.net_create_status), ('shared', False)]
-        #
-        # with mock.patch(HTTPCON) as conmock:
-        #     rv = conmock.return_value
-        #     rv.getresponse.return_value = HTTPResponseMock(None)
-        #     with self.network(name=name) as net:
-        #         # for debug
-        #         print (rv.request.mock_calls)
-        #         network = jsonutils.loads(rv.request.mock_calls[0][1][2])
-        #         self.assertIn('tenant_name', network['network'])
-        #         self.assertEqual('tenant_name',
-        #                          network['network']['tenant_name'])
-        #         for k, v in keys:
-        #             self.assertEqual(net['network'][k], v)
+        name = 'net1'
+        keys = [('subnets', []), ('name', name), ('admin_state_up', True),
+                ('status', self.net_create_status), ('shared', False)]
+
+        with mock.patch(HTTPCON) as conmock:
+            rv = conmock.return_value
+            rv.getresponse.return_value = HTTPResponseMock(None)
+            with self.network(name=name) as net:
+                # for debug
+                print(rv.request.mock_calls)
+
+                # remove topo sync call
+                filtered_calls = \
+                    list(filter(lambda call: "topology" not in call[1][1],
+                                rv.request.mock_calls))
+
+                network = jsonutils.loads(filtered_calls[0][1][2])
+                self.assertIn('tenant_name', network['network'])
+                self.assertEqual('tenant_name',
+                                 network['network']['tenant_name'])
+                for k, v in keys:
+                    self.assertEqual(net['network'][k], v)
 
     def test_update_network(self):
         with self.network() as network:
@@ -144,9 +142,13 @@ class TestBigSwitchML2Router(test_l3.L3NatTestCaseBase,
         # TODO(Joe): This test fails. Why?
         pass
 
+    def test_router_add_gateway_notifications(self):
+        # This creates a gateway port without ip, not supported
+        pass
+
     def test_router_add_interface_by_port_fails_nested(self):
         # Force _validate_router_port_info failure
-        plugin = directory.get_plugin(lib_constants.L3)
+        plugin = directory.get_plugin(plugin_constants.L3)
         if not isinstance(plugin, l3_db.L3_NAT_dbonly_mixin):
             self.skipTest("Plugin is not L3_NAT_dbonly_mixin")
         orig_update_port = self.plugin.update_port
@@ -265,7 +267,8 @@ class TestBigSwitchMechDriverPortsV2(test_db_base_plugin_v2.TestPortsV2,
         host_arg = {portbindings.HOST_ID: 'hostname',
                     portbindings.VNIC_TYPE: portbindings.VNIC_DIRECT}
         with\
-            mock.patch(SERVER_POOL + '.rest_get_switch', return_value=True) as rmock,\
+            mock.patch(SERVER_POOL + '.rest_get_switch',
+                       return_value=True) as rmock,\
             self.port(arg_list=(portbindings.HOST_ID, portbindings.VNIC_TYPE),
                       **host_arg):
 
